@@ -6,7 +6,7 @@
 #' This function is built into the class \link{omics} with method \code{alpha_diversity()} and inherited by other omics classes, such as;
 #' \link{metagenomics} and \link{proteomics}.
 #'
-#' @param x A \link[base]{matrix} or \link[Matrix]{sparseMatrix}.
+#' @param x A \link[base]{matrix}, \link[Matrix]{sparseMatrix} or \link[Matrix]{Matrix}.
 #' @param metric A character variable for metric; shannon, simpson or invsimpson.
 #' @param normalize A boolean variable for sample normalization by column sums.
 #' @param base Input for \link[base]{log} to use natural logarithmic scale, log2, log10 or other.
@@ -48,10 +48,10 @@ diversity <- function(x,
   ## Error handling
   #--------------------------------------------------------------------#
 
-  if (is.vector(x))
-    cli::cli_abort("Input must a matrix of class matrix or Matrix, not a vector.")
+  if (inherits(x, "denseMatrix") || inherits(x, "matrix") || inherits(x, "sparseMatrix")) {
+    x <- as(x, "CsparseMatrix")
+  } else cli::cli_abort("Input isn't a {.cls matrix}, {.cls denseMatrix} or {.cls sparseMatrix}.")
 
-  x <- drop(as(x, "sparseMatrix"))
   if (!is.numeric(x@x))
     cli::cli_abort("Input data must be numeric")
 
@@ -60,14 +60,13 @@ diversity <- function(x,
 
   OPTIONS <- c("shannon", "simpson", "invsimpson")
   if (!is.character(metric) && length(metric) != 1) {
-    cli::cli_abort("{metric} needs to contain characters with length of 1.")
+    cli::cli_abort("{.val {metric}} needs to contain characters with length of 1.")
   } else if (!metric %in% OPTIONS) {
-    cli::cli_abort("{metric} is not a valid metric. Valid options: {OPTIONS}")
+    cli::cli_abort("{.val {metric}} is not a valid metric. Valid options: <{.val {OPTIONS}}>")
   }
 
   ## MAIN
-  #--------------------------------------------------------------------#
-
+  #--------------------------------------------------------------------#    
   total <- rep(Matrix::colSums(x), base::diff(x@p))
   if (normalize) {
     x@x <- x@x / total
