@@ -15,29 +15,48 @@
 
 metagenomics <- R6::R6Class(
   classname = "metagenomics",
-  cloneable = FALSE,
+  cloneable = TRUE,
   inherit = omics,
+  active = list(
+    #' @field treeData A "phylo" class, see \link[ape]{as.phylo}.
+    treeData = function(value) {
+      # back-up
+      .countData <- private$.countData
+      .featureData <- private$.featureData
+      .metaData <- private$.metaData
+      .treeData <- private$.treeData
+
+      # restore on error
+      success <- FALSE
+      on.exit({
+        if (!success) {
+          private$.countData <- .countData
+          private$.featureData <- .featureData
+          private$.metaData <- .metaData
+          private$.treeData <- .treeData
+        }
+      }, add = TRUE)
+
+      if (missing(value)) {
+        success <- TRUE
+        private$.treeData
+      } else if (inherits(value, "phylo")) {
+        private$.treeData <- value
+        private$sync()
+        self$print()
+        success <- TRUE
+        invisible(self)
+      } else {
+        cli::cli_abort("Data input must be {.cls phylo} like {.field treeData}.")
+      }
+    }
+  ),
   public = list(
-    #' @field countData A path to an existing file, data.table, data.frame, matrix or sparseMatrix with zero values.
-    countData = NULL,
-
-    #' @field metaData A path to an existing file, data.table or data.frame.
-    metaData = NULL,
-
-    #' @field featureData A path to an existing file, data.table or data.frame.
-    featureData = NULL,
-
-    #' @field treeData A path to an existing newick file or class "phylo", see \link[ape]{read.tree}.
-    treeData = NULL,
-
-    #' @field biomData A path to an existing biom file or hdf5 file, see \link[rhdf5]{h5read}.
-    biomData = NULL,
-
     #' @description
     #' Initializes the metagenomics class object with \code{metagenomics$new()}
-    #' @param countData countData A path to an existing file, data.table, data.frame, matrix or sparseMatrix with zero values.
-    #' @param featureData A path to an existing file, data.table or data.frame.
-    #' @param metaData A path to an existing file, data.table or data.frame.
+    #' @param countData A path to an existing file or a dense/sparse \link[Matrix]{Matrix} format.
+    #' @param featureData A path to an existing file, \link[data.table]{data.table} or data.frame.
+    #' @param metaData A path to an existing file, \link[data.table]{data.table} or data.frame.
     #' @param treeData A path to an existing newick file or class "phylo", see \link[ape]{read.tree}.
     #' @param biomData A path to an existing biom file, version 2.1.0 (http://biom-format.org/), see \link[rhdf5]{h5read}.
     #' @param feature_names A character vector to name the feature names that fit the supplied `featureData`.
@@ -92,7 +111,7 @@ metagenomics <- R6::R6Class(
             }
 
             # Loads data in memory
-            self$biomData <- rhdf5::h5read(biomData, "/", read.attributes = TRUE)
+            private$.biomData <- rhdf5::h5read(biomData, "/", read.attributes = TRUE)
             private$construct_hdf5_featureData()
             private$construct_hdf5_countData()
 
@@ -102,13 +121,30 @@ metagenomics <- R6::R6Class(
 
           } else if (yyjsonr::validate_json_file(biomData)) {
             
-            self$biomData <- jsonlite::read_json(biomData)
+            private$.biomData <- jsonlite::read_json(biomData)
             private$construct_json_featureData(feature_names)
             private$construct_json_countData()
 
           } else {
-            cli::cli_abort("biomData could not be loaded. Not a valid JSON or HDF5 format!")
+            cli::cli_abort("{.field biomData} could not be loaded. Not a valid JSON or HDF5 format!")
           }
+
+          # Create placeholder featureData
+          if (is.null(private$.featureData)) {
+            FEATURE_ID <- paste0("feature_", 1:nrow(private$.countData))
+            private$.featureData <- data.table::data.table()
+            private$.featureData <- private$.featureData[, (private$.feature_id) := FEATURE_ID]
+            rownames(private$.countData) <- private$.featureData[[ private$.feature_id ]]
+            cli::cli_alert_warning("Created placeholder {.field featureData}.")
+          }
+          rownames(private$.countData) <- private$.featureData[[ private$.feature_id ]]
+          data.table::setcolorder(
+            x = private$.featureData,
+            neworder = c(private$.feature_id, base::setdiff(colnames(private$.featureData), private$.feature_id))
+          )
+          private$.featureData <- private$.featureData[, 
+            lapply(.SD, function(x) ifelse(x == "", NA, x)),
+            .SDcols = colnames(private$.featureData)]
         }
       }
 
@@ -118,18 +154,18 @@ metagenomics <- R6::R6Class(
 
       if (!is.null(treeData)) {
         if (is.character(treeData) && length(treeData) == 1 && file.exists(treeData)) {
-          self$treeData <- ape::read.tree(treeData)
-          cli::cli_alert_success("treeData is loaded.")
+          private$.treeData <- ape::read.tree(treeData)
+          cli::cli_alert_success("{.field treeData} is loaded.")
         } else if (inherits(treeData, "phylo")) {
-          self$treeData <- treeData
-          cli::cli_alert_success("treeData is loaded.")
+          private$.treeData <- treeData
+          cli::cli_alert_success("{.field treeData} is loaded.")
         } else {
-          cli::cli_alert_warning("The provided TreeData could not be loaded. Make sure the tree is supported by `ape::read.tree`")
+          cli::cli_alert_warning("The provided {.field treeData} could not be loaded. Make sure the tree is supported by {.fun ape::read.tree}")
         }
 
         # Aligning featureData and countData rows by tree tips
-        self$featureData <- self$featureData[order(match(self$featureData$FEATURE_ID, self$treeData$tip.label))]
-        self$countData <- self$countData[self$featureData$FEATURE_ID, ]
+        private$.featureData <- private$.featureData[order(match(private$.featureData[[ private$.feature_id ]], private$.treeData$tip.label))]
+        private$.countData <- private$.countData[private$.featureData[[ private$.feature_id ]], ]
       }
 
       #-------------------#
@@ -139,126 +175,26 @@ metagenomics <- R6::R6Class(
       cli::cli_alert_info("Final steps .. cleaning & creating back-up")
 
       # Removing prefix of taxonomic features
-      self$featureData <- self$featureData[, lapply(.SD, function(x) gsub("^[dpcofgs]_{2}", "", x)),
-                                           .SDcols = colnames(self$featureData)]
+      private$.featureData <- private$.featureData[, lapply(.SD, function(x) gsub("^[dpcofgs]_{2}", "", x)),
+                                           .SDcols = colnames(private$.featureData)]
       # Rename last column names by feature_names
       n_feature_names <- length(feature_names)
-      n_cols_featureData <- ncol(self$featureData)
-      colnames(self$featureData)[n_cols_featureData:(n_cols_featureData - n_feature_names + 1)] <- base::rev(feature_names)
+      n_cols_featureData <- ncol(private$.featureData)
+      colnames(private$.featureData)[n_cols_featureData:(n_cols_featureData - n_feature_names + 1)] <- base::rev(feature_names)
 
       # Subsetting countData by metadata
-      self$countData <- self$countData[, self$metaData[[ self$.sample_id ]], drop = FALSE]
-
+      private$sync()
       self$print()
 
       # saves data for reset function
       private$original_data = list(
-        counts = self$countData,
-        features = self$featureData,
-        metadata = self$metaData,
-        tree = self$treeData
+        counts = private$.countData,
+        features = private$.featureData,
+        metadata = private$.metaData,
+        tree = private$.treeData
       )
     },
     #' @description
-    #' Displays parameters of the metagenomics object via stdout.
-    #' @examples
-    #' library("OmicFlow")
-    #'
-    #' metadata_file <- system.file("extdata", "metadata.tsv", package = "OmicFlow")
-    #' counts_file <- system.file("extdata", "counts.tsv", package = "OmicFlow")
-    #' features_file <- system.file("extdata", "features.tsv", package = "OmicFlow")
-    #' tree_file <- system.file("extdata", "tree.newick", package = "OmicFlow")
-    #'
-    #' taxa <- metagenomics$new(
-    #'  metaData = metadata_file,
-    #'  countData = counts_file,
-    #'  featureData = features_file,
-    #'  treeData = tree_file
-    #' )
-    #'
-    #' # method 1 to call print function
-    #' taxa
-    #'
-    #' # method 2 to call print function
-    #' taxa$print()
-    #'
-    #' @return object in place
-    print = function() {
-      cat("## metagenomics-class object \n")
-      if (length(self$countData) > 0) cat(paste0("## countData:\t[ ", ncol(self$countData), " Samples and ", nrow(self$countData), " Features\t] \n"))
-      if (length(self$metaData) > 0) cat(paste0("## metaData:\t[ ", ncol(self$metaData), " Variables and ", nrow(self$metaData), " Samples\t] \n"))
-      if (length(self$featureData) > 0) cat(paste0("## taxData:\t[ ", ncol(self$featureData)-1, " Ranks and ", nrow(self$featureData), " Taxa\t] \n"))
-      if (length(self$treeData) > 0) cat(paste0("## treeData:\t[ ", length(self$treeData$tip.label), " Tips and ", self$treeData$Nnode, " Nodes\t] \n"))
-    },
-    #' @description
-    #' Upon creation of a new `metagenomics` object a small backup of the original data is created.
-    #' Since modification of the object is done by reference and duplicates are not made, it is possible to `reset` changes to the class.
-    #' The methods from the abstract class \link{omics} also contains a private method to prevent any changes to the original object when using methods such as \code{ordination} \code{alpha_diversity} or \code{$DFE}.
-    #' @examples
-    #' library(ggplot2)
-    #' library("OmicFlow")
-    #'
-    #' metadata_file <- system.file("extdata", "metadata.tsv", package = "OmicFlow")
-    #' counts_file <- system.file("extdata", "counts.tsv", package = "OmicFlow")
-    #' features_file <- system.file("extdata", "features.tsv", package = "OmicFlow")
-    #' tree_file <- system.file("extdata", "tree.newick", package = "OmicFlow")
-    #'
-    #' taxa <- metagenomics$new(
-    #'  metaData = metadata_file,
-    #'  countData = counts_file,
-    #'  featureData = features_file,
-    #'  treeData = tree_file
-    #' )
-    #'
-    #' # Performs modifications
-    #' taxa$transform(log2)
-    #'
-    #' # resets
-    #' taxa$reset()
-    #'
-    #' # An inbuilt reset function prevents unwanted modification to the taxa object.
-    #' taxa$rankstat(feature_ranks = c("Kingdom", "Phylum", "Family", "Genus", "Species"))
-    #'
-    #' @return object in place
-    reset = function() {
-      self$countData = private$original_data$counts
-      self$featureData = private$original_data$features
-      self$metaData = private$original_data$metadata
-      self$treeData = private$original_data$tree
-      invisible(self)
-    },
-    #' @description
-    #' Removes empty (zero) values by row, column and tips from the `countData` and `treeData`.
-    #' This method is performed automatically during subsetting of the object.
-    #' @importFrom ape keep.tip
-    #' @examples
-    #' library("OmicFlow")
-    #'
-    #' metadata_file <- system.file("extdata", "metadata.tsv", package = "OmicFlow")
-    #' counts_file <- system.file("extdata", "counts.tsv", package = "OmicFlow")
-    #' features_file <- system.file("extdata", "features.tsv", package = "OmicFlow")
-    #' tree_file <- system.file("extdata", "tree.newick", package = "OmicFlow")
-    #'
-    #' taxa <- metagenomics$new(
-    #'  metaData = metadata_file,
-    #'  countData = counts_file,
-    #'  featureData = features_file,
-    #'  treeData = tree_file
-    #' )
-    #' 
-    #' # Sample subset induces empty features
-    #' taxa$sample_subset(treatment == "tumor")
-    #'
-    #' # Remove empty features from countData and treeData
-    #' taxa$removeZeros()
-    #' 
-    #' @return object in place
-    removeZeros = function() {
-      super$removeZeros()
-      if (!is.null(self$treeData)) self$treeData <- ape::keep.tip(self$treeData, self$featureData$FEATURE_ID)
-      invisible(self)
-    },
-    #' @description
     #' Creates a BIOM file in HDF5 format of the loaded items via ['new()'](#method-new), which is compatible to the python biom-format version 2.1, see http://biom-format.org.
     #' @param filename A character variable of either the full path of filename of the biom file (e.g. `output.biom`)
     #' @examples
@@ -313,7 +249,7 @@ metagenomics <- R6::R6Class(
         cli::cli_abort("Can't open HDF5 file {.filename {filename}}: {h5}")
 
       # convert countData to triplet matrix
-      triplets <- Matrix::summary(self$countData)
+      triplets <- Matrix::summary(private$.countData)
 
       #----------------------------#
       #       Add Attributes       #
@@ -336,7 +272,7 @@ metagenomics <- R6::R6Class(
       rhdf5::h5writeAttribute(attr = paste(Sys.Date()),
                               h5obj = h5,
                               name = 'creation-date')
-      rhdf5::h5writeAttribute(attr = dim(self$countData),
+      rhdf5::h5writeAttribute(attr = dim(private$.countData),
                               h5obj = h5,
                               name = 'shape')
       rhdf5::h5writeAttribute(attr = length(triplets),
@@ -352,10 +288,10 @@ metagenomics <- R6::R6Class(
       x <- matrix(c(triplets$i - 1, triplets$j - 1, triplets$x), ncol = 3)
       x <- x[order(x[,1]),,drop=FALSE]
 
-      counts_per_row <- base::tabulate(x[,1] + 1L, nbins = nrow(self$countData))
+      counts_per_row <- base::tabulate(x[,1] + 1L, nbins = nrow(private$.countData))
       indptr <- c(0L, base::cumsum(counts_per_row))
 
-      rhdf5::h5writeDataset(obj = base::rownames(self$countData),
+      rhdf5::h5writeDataset(obj = base::rownames(private$.countData),
                             h5loc = h5,
                             name = 'observation/ids')
       rhdf5::h5writeDataset(obj = as.numeric(x[,3]),
@@ -372,10 +308,10 @@ metagenomics <- R6::R6Class(
       #       Counts by column     #
       #----------------------------#
       x <- x[order(x[,2]),,drop=FALSE]
-      counts_per_col <- base::tabulate(x[,2] + 1L, nbins = ncol(self$countData))
+      counts_per_col <- base::tabulate(x[,2] + 1L, nbins = ncol(private$.countData))
       indptr <- c(0L, cumsum(counts_per_col))
 
-      rhdf5::h5writeDataset(obj = base::colnames(self$countData),
+      rhdf5::h5writeDataset(obj = base::colnames(private$.countData),
                             h5loc = h5,
                             name = 'sample/ids')
       rhdf5::h5writeDataset(obj = as.numeric(x[,3]),
@@ -391,9 +327,9 @@ metagenomics <- R6::R6Class(
       #----------------------------#
       #       Add Taxonomy         #
       #----------------------------#
-      if (all(dim(self$featureData)) > 0) {
+      if (all(dim(private$.featureData)) > 0) {
         h5path <- 'observation/metadata/taxonomy'
-        features <- as.matrix(self$featureData[, .SD, .SDcols = !c("FEATURE_ID")])
+        features <- as.matrix(private$.featureData[, .SD, .SDcols = !c(private$.feature_id)])
         dimnames(features) <- list(NULL, NULL)
         rhdf5::h5writeDataset(obj = features,
                               h5loc = h5,
@@ -405,56 +341,65 @@ metagenomics <- R6::R6Class(
     }
   ),
   private = list(
+    # Private data fields
+    #-------------------------#
+    .countData = NULL,
+    .featureData = NULL,
+    .metaData = NULL,
+    .treeData = NULL,
+    .biomData = NULL,
+    .feature_id = "FEATURE_ID",
+    .sample_id = "SAMPLE_ID",
+    .samplepair_id = "SAMPLEPAIR_ID",
     original_data = list(),
     construct_hdf5_featureData = function() {
-      self$featureData <- data.table::data.table(t(self$biomData$observation$metadata$taxonomy))
-
-      if (any(grepl(self$.feature_id, colnames(self$metaData))) && !all(is.na(self$metaData[[ self$.feature_id ]]))) {
-        FEATURE_ID <- self$metaData[[self$.feature_id]]
+      if (!is.null(private$.biomData$observation$metadata$taxonomy)) {
+        private$.featureData <- data.table::data.table(t(private$.biomData$observation$metadata$taxonomy))
+      }
+      
+      if (any(grepl(private$.feature_id, colnames(private$.metaData))) && !all(is.na(private$.metaData[[ private$.feature_id ]]))) {
+        FEATURE_ID <- private$.metaData[[private$.feature_id]]
+      } else if (!is.null(private$.biomData$observation$ids)) {
+        FEATURE_ID <- private$.biomData$observation$ids
       } else {
-        FEATURE_ID <- self$biomData$observation$ids
+        FEATURE_ID <- NULL
       }
 
       # Adds feature id as first column
-      self$featureData[[ self$.feature_id ]] <- FEATURE_ID
-      data.table::setcolorder(x = self$featureData,
-                              neworder = c(self$.feature_id, base::setdiff(colnames(self$featureData), self$.feature_id))
-                              )
-      colnames(self$featureData) <- gsub("\\s+", "_", colnames(self$featureData))
-      self$featureData <- self$featureData[, lapply(.SD, function(x) ifelse(x == "", NA, x)),
-                                           .SDcols = colnames(self$featureData)]
+      if (!is.null(FEATURE_ID) && !is.null(private$.featureData)) {
+        private$.featureData[[ private$.feature_id ]] <- FEATURE_ID
+      }
 
-      cli::cli_alert_success("featureData is loaded.")
+      cli::cli_alert_success("{.field featureData} is loaded.")
     },
     construct_hdf5_countData = function() {
-      indptr <- as.numeric(self$biomData$observation$matrix$indptr)
+      indptr <- as.numeric(private$.biomData$observation$matrix$indptr)
 
-      self$countData <- Matrix::sparseMatrix(
+      private$.countData <- Matrix::sparseMatrix(
         i        = unlist(sapply(1:(length(indptr)-1), function (i) rep(i, diff(indptr[c(i,i+1)])))),
-        j        = as.numeric(self$biomData$observation$matrix$indices) + 1,
-        x        = as.numeric(self$biomData$observation$matrix$data),
-        dims     = c(length(self$biomData$observation$ids), length(self$biomData$sample$ids)),
+        j        = as.numeric(private$.biomData$observation$matrix$indices) + 1,
+        x        = as.numeric(private$.biomData$observation$matrix$data),
+        dims     = c(length(private$.biomData$observation$ids), length(private$.biomData$sample$ids)),
         dimnames = list(
-          as.character(self$biomData$observation$ids),
-          as.character(self$biomData$sample$ids)
+          as.character(private$.biomData$observation$ids),
+          as.character(private$.biomData$sample$ids)
         ))
 
-      rownames(self$countData) <- self$featureData$FEATURE_ID
-
-      cli::cli_alert_success("countData is loaded.")
+      cli::cli_alert_success("{.field countData} is loaded.")
     },
     construct_json_featureData = function(feature_names) {
       # Create empty featureData
-      self$featureData <- data.table::data.table(matrix(NA_character_,
-                                                        nrow = length(self$biomData$rows),
-                                                        ncol = length(c(self$.feature_id, feature_names))))
-      setNames(self$featureData, c(self$.feature_id, feature_names))
+      private$.featureData <- data.table::data.table(matrix(NA_character_,
+                                                     nrow = length(private$.biomData$rows),
+                                                     ncol = length(c(private$.feature_id, feature_names))
+                                                     ))
+      setNames(private$.featureData, c(private$.feature_id, feature_names))
 
       # Fill first column with $id values
-      self$featureData[["FEATURE_ID"]] <- vapply(self$biomData$rows, function(x) as.character(x$id), character(1))
+      private$.featureData[[ private$.feature_id ]] <- vapply(private$.biomData$rows, function(x) as.character(x$id), character(1))
 
-      for (i in seq_along(self$biomData$rows)) {
-        taxonomy <- self$biomData$rows[[i]]$metadata$taxonomy
+      for (i in seq_along(private$.biomData$rows)) {
+        taxonomy <- private$.biomData$rows[[i]]$metadata$taxonomy
 
         # Skip if taxonomy is missing or NULL
         if (is.null(taxonomy)) next
@@ -465,40 +410,30 @@ metagenomics <- R6::R6Class(
 
         # Fill featureData with tax values by index
         col_positions <- tax_indices + 1
-        self$featureData[i, (col_positions) := as.list(tax_values)]
+        private$.featureData[i, (col_positions) := as.list(tax_values)]
       }
 
-      if (any(grepl(self$.feature_id, colnames(self$metaData))) && !all(is.na(self$metaData[[self$.feature_id]]))) {
-        FEATURE_ID <- self$metaData[[self$.feature_id]]
+      if (any(grepl(private$.feature_id, colnames(private$.metaData))) && !all(is.na(private$.metaData[[private$.feature_id]]))) {
+        FEATURE_ID <- private$.metaData[[private$.feature_id]]
+        private$.featureData[, (private$.feature_id) := FEATURE_ID]
       }
-
-      # Adds feature id as first column
-      self$featureData[, (self$.feature_id) := FEATURE_ID]
-      data.table::setcolorder(x = self$featureData,
-                              neworder = c(self$.feature_id, base::setdiff(colnames(self$featureData), self$.feature_id))
-      )
-      colnames(self$featureData) <- gsub("\\s+", "_", colnames(self$featureData))
-
-      cli::cli_alert_success("featureData is loaded.")
+      cli::cli_alert_success("{.field featureData} is loaded.")
     },
     construct_json_countData = function() {
-      feature_ids <- sapply(self$biomData$rows, function(x) unlist(x$id))
-      sample_ids <- sapply(self$biomData$columns, function(x) unlist(x$id))
+      feature_ids <- sapply(private$.biomData$rows, function(x) unlist(x$id))
+      sample_ids <- sapply(private$.biomData$columns, function(x) unlist(x$id))
 
-      self$countData <- Matrix::sparseMatrix(
-        i        = sapply(self$biomData$data, function(x) x[[1]]) + 1,
-        j        = sapply(self$biomData$data, function(x) x[[2]]) + 1,
-        x        = sapply(self$biomData$data, function(x) x[[3]]),
+      private$.countData <- Matrix::sparseMatrix(
+        i        = sapply(private$.biomData$data, function(x) x[[1]]) + 1,
+        j        = sapply(private$.biomData$data, function(x) x[[2]]) + 1,
+        x        = sapply(private$.biomData$data, function(x) x[[3]]),
         dims     = c( length(feature_ids), length(sample_ids) ),
         dimnames = list(
           as.character(feature_ids),
           as.character(sample_ids)
         )
       )
-
-      rownames(self$countData) <- self$featureData$FEATURE_ID
-
-      cli::cli_alert_success("countData is loaded.")
+      cli::cli_alert_success("{.field countData} is loaded.")
     }
   )
 )
